@@ -144,7 +144,7 @@ tool_rc tpm2_policy_build_pcr(ESYS_CONTEXT *ectx, tpm2_session *policy_session,
         // Read PCRs
         tool_rc rc = tpm2_pcr_read(ectx, ESYS_TR_NONE, ESYS_TR_NONE,
                 ESYS_TR_NONE, pcr_selections, &pcr_update_counter,
-                NULL, &pcr_val);
+                NULL, &pcr_val, NULL, TPM2_ALG_ERROR);
         if (rc != tool_rc_success) {
             return rc;
         }
@@ -293,7 +293,8 @@ tool_rc tpm2_policy_build_policysecret(ESYS_CONTEXT *ectx,
         tpm2_session *policy_session, tpm2_loaded_object *auth_entity_obj,
         INT32 expiration, TPMT_TK_AUTH **policy_ticket,
         TPM2B_TIMEOUT **timeout, bool is_nonce_tpm,
-        const char *policy_qualifier_data, TPM2B_DIGEST *cp_hash) {
+        const char *policy_qualifier_data, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm) {
 
     /*
      * Qualifier data is optional. If not specified default to 0
@@ -322,7 +323,7 @@ tool_rc tpm2_policy_build_policysecret(ESYS_CONTEXT *ectx,
 
     rc = tpm2_policy_secret(ectx, auth_entity_obj, policy_session_handle,
         expiration, policy_ticket, timeout, nonce_tpm, &policy_qualifier,
-        cp_hash);
+        cp_hash, parameter_hash_algorithm);
 
 tpm2_policy_build_policysecret_out:
     Esys_Free(nonce_tpm);
@@ -490,12 +491,13 @@ tpm2_policy_build_policysigned_out:
 }
 
 tool_rc tpm2_policy_get_digest(ESYS_CONTEXT *ectx, tpm2_session *session,
-        TPM2B_DIGEST **policy_digest) {
+    TPM2B_DIGEST **policy_digest, TPM2B_DIGEST *cphash,
+    TPMI_ALG_HASH parameter_hash_algorithm) {
 
     ESYS_TR handle = tpm2_session_get_handle(session);
 
     return tpm2_policy_getdigest(ectx, handle, ESYS_TR_NONE, ESYS_TR_NONE,
-            ESYS_TR_NONE, policy_digest);
+            ESYS_TR_NONE, policy_digest, cphash, parameter_hash_algorithm);
 }
 
 tool_rc tpm2_policy_build_policycommandcode(ESYS_CONTEXT *ectx,
@@ -655,8 +657,9 @@ bool tpm2_policy_parse_policy_list(char *str, TPML_DIGEST *policy_list) {
 tool_rc tpm2_policy_tool_finish(ESYS_CONTEXT *ectx, tpm2_session *session,
         const char *save_path) {
 
-    TPM2B_DIGEST *policy_digest = NULL;
-    tool_rc rc = tpm2_policy_get_digest(ectx, session, &policy_digest);
+    TPM2B_DIGEST *policy_digest = 0;
+    tool_rc rc = tpm2_policy_get_digest(ectx, session, &policy_digest, 0,
+        TPM2_ALG_ERROR);
     if (rc != tool_rc_success) {
         LOG_ERR("Could not build tpm policy");
         return rc;
