@@ -49,6 +49,12 @@ setup_owner_policy() {
     tpm2 policycphash -S session.ctx --cphash-input cp.hash
 }
 
+setup_platform_policy() {
+    tpm2 setprimarypolicy -C p -L policy.cphash -g sha256
+    tpm2 startauthsession -S session.ctx --policy-session -g sha256
+    tpm2 policycphash -S session.ctx --cphash-input cp.hash
+}
+
 start_policy_cphash() {
     tpm2 startauthsession -S session.ctx --policy-session -g sha256
     tpm2 policycphash -S session.ctx --cphash-input cp.hash
@@ -141,10 +147,22 @@ tpm2 nvundefine 1
 # Test tpm2 nvincrement
 create_authorized_policy
 tpm2 nvdefine 1 -s 8 -a "nt=counter|ownerread|policywrite" -L authorized.policy
-tpm2 nvincrement 1 --cphash cp.hash
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+#
+# Suppose TPM2_NVDefine was not run and the name was entirely created external
+# to the TPM. Note: This support still needs to be added.
+#
+# To emulate the absence of NV index while calculating the cpHash of NVRead,
+# let's Undefine the NV index and define it later.
+#
+tpm2 nvundefine 1 -C o
+tpm2 nvincrement 1 --cphash cp.hash -n $NV_INDEX_NAME --tcti=none
 generate_policycphash
 sign_and_verify_policycphash
+
+tpm2 nvdefine 1 -s 8 -a "nt=counter|ownerread|policywrite" -L authorized.policy
 setup_authorized_policycphash
+
 tpm2 nvincrement 1 -P "session:session.ctx"
 tpm2 flushcontext session.ctx
 tpm2 nvundefine 1
@@ -190,9 +208,21 @@ tpm2 nvundefine 1 -C o
 create_authorized_policy
 tpm2 nvdefine 1 -C o -s 32 -a "policyread|policywrite|read_stclear" \
 -L authorized.policy
-tpm2 nvreadlock 1 -C 0x01000001 --cphash cp.hash
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+#
+# Suppose TPM2_NVDefine was not run and the name was entirely created external
+# to the TPM. Note: This support still needs to be added.
+#
+# To emulate the absence of NV index while calculating the cpHash of NVRead,
+# let's Undefine the NV index and define it later.
+#
+tpm2 nvundefine 1 -C o
+tpm2 nvreadlock 1 -C 0x01000001 --cphash cp.hash -n $NV_INDEX_NAME --tcti=none
 generate_policycphash
 sign_and_verify_policycphash
+
+tpm2 nvdefine 1 -C o -s 32 -a "policyread|policywrite|read_stclear" \
+-L authorized.policy
 setup_authorized_policycphash
 tpm2 nvreadlock 1 -C 0x01000001 -P "session:session.ctx"
 tpm2 flushcontext session.ctx
@@ -202,24 +232,49 @@ tpm2 nvundefine 1
 create_authorized_policy
 tpm2 nvdefine 1 -C o -s 32 -a "policyread|policywrite|writedefine" \
 -L authorized.policy
-tpm2 nvwritelock 1 -C 0x01000001 --cphash cp.hash
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+#
+# Suppose TPM2_NVDefine was not run and the name was entirely created external
+# to the TPM. Note: This support still needs to be added.
+#
+# To emulate the absence of NV index while calculating the cpHash of NVRead,
+# let's Undefine the NV index and define it later.
+#
+tpm2 nvundefine 1 -C o
+tpm2 nvwritelock 1 -C 0x01000001 --cphash cp.hash -n $NV_INDEX_NAME --tcti=none
 generate_policycphash
 sign_and_verify_policycphash
+
+tpm2 nvdefine 1 -C o -s 32 -a "policyread|policywrite|writedefine" \
+-L authorized.policy
 setup_authorized_policycphash
 tpm2 nvwritelock 1 -C 0x01000001 -P "session:session.ctx"
 tpm2 flushcontext session.ctx
 tpm2 nvundefine 1
+
 ## attempt with globallock attribute set
 tpm2 nvdefine 1 -C o -s 32 -a "ownerread|ownerwrite|globallock"
-tpm2 nvwritelock --global -C o --cphash cp.hash
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+#
+# Suppose TPM2_NVDefine was not run and the name was entirely created external
+# to the TPM. Note: This support still needs to be added.
+#
+# To emulate the absence of NV index while calculating the cpHash of NVRead,
+# let's Undefine the NV index and define it later.
+#
+tpm2 nvundefine 1 -C o
+tpm2 nvwritelock --global -C o --cphash cp.hash --tcti=none
 generate_policycphash
 setup_owner_policy
+
+tpm2 nvdefine 1 -C o -s 32 -a "ownerread|ownerwrite|globallock"
 tpm2 nvwritelock --global -C o -P "session:session.ctx"
 tpm2 flushcontext session.ctx
 tpm2 nvundefine 1
 
 # Test tpm2 nvdefine
-tpm2 nvdefine 1 -C o -s 32 -a "ownerread|ownerwrite" --cphash cp.hash
+tpm2 nvdefine 1 -C o -s 32 -a "ownerread|ownerwrite" --cphash cp.hash \
+  --tcti=none
 generate_policycphash
 setup_owner_policy
 tpm2 nvdefine 1 -C o -s 32 -a "ownerread|ownerwrite" -P "session:session.ctx"
@@ -238,9 +293,12 @@ tpm2 nvundefine 1
 
 # Test tpm2 nvundefine
 tpm2 nvdefine 1
-tpm2 nvundefine 1 --cphash cp.hash
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+tpm2 nvundefine 1
+tpm2 nvundefine 1 --cphash cp.hash --tcti=none -n $NV_INDEX_NAME
 generate_policycphash
 setup_owner_policy
+tpm2 nvdefine 1
 tpm2 nvundefine 1 -P "session:session.ctx"
 tpm2 flushcontext session.ctx
 ## attempt failing scenario
@@ -256,6 +314,41 @@ trap onerror ERR
 tpm2 flushcontext session.ctx
 tpm2 nvundefine -C p 2
 
+# Test tpm2 nvundefine - spacespecial
+#
+# Get NV index name
+#
+tpm2 startauthsession -S special_session.ctx --policy-session
+tpm2 policycommandcode -S special_session.ctx TPM2_CC_NV_UndefineSpaceSpecial \
+-L special_policy.digest
+tpm2 nvdefine 1 -C p -s 32 -a "authread|authwrite|policydelete|platformcreate" \
+-L special_policy.digest
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+tpm2 nvundefine 1 -S special_session.ctx -C p
+tpm2 flushcontext special_session.ctx
+#
+# Get cpHash with tcti=none
+#
+tpm2 nvundefine 1 -n $NV_INDEX_NAME --with-policydelete --cphash cp.hash \
+--tcti=none
+generate_policycphash
+setup_platform_policy
+#
+# Change platform auth to ensure using auth policy for authentication.
+#
+tpm2 nvdefine 1 -C p -s 32 -a "authread|authwrite|policydelete|platformcreate" \
+-L special_policy.digest
+tpm2 startauthsession -S special_session.ctx --policy-session
+tpm2 policycommandcode -S special_session.ctx TPM2_CC_NV_UndefineSpaceSpecial
+#
+# Auth platform with auth session this time
+#
+tpm2 changeauth -c p platformauth
+tpm2 nvundefine 1 -S special_session.ctx -P session:session.ctx
+tpm2 flushcontext session.ctx
+tpm2 flushcontext special_session.ctx
+tpm2 changeauth -c p -p platformauth
+
 #Test tpm2 nvcertify
 tpm2 createprimary -C o -c primary.ctx -Q
 tpm2 create -G rsa -u signing_key.pub -r signing_key.priv -C primary.ctx \
@@ -263,8 +356,12 @@ tpm2 create -G rsa -u signing_key.pub -r signing_key.priv -C primary.ctx \
 tpm2 readpublic -c signing_key.ctx -f pem -o sslpub.pem -Q
 tpm2 nvdefine -s 32 -C o -a "ownerread|ownerwrite|authread|authwrite" 1
 dd if=/dev/urandom bs=1 count=32 status=none| tpm2 nvwrite 1 -i-
-tpm2 nvcertify -C signing_key.ctx -g sha256 -f plain -s rsassa \
--o signature.bin --attestation attestation.bin --size 32 1 -c o --cphash cp.hash
+
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+SIGNER_NAME=$(tpm2 readpublic -c signing_key.ctx | grep name | grep -v -e q -e g | awk {'print $2'})
+
+tpm2 nvcertify -s rsassa -g sha256 --size 32 1 --cphash cp.hash --tcti none \
+--name $NV_INDEX_NAME --signer-name $SIGNER_NAME -c o
 generate_policycphash
 setup_owner_policy
 tpm2 nvcertify -C signing_key.ctx -g sha256 -f plain -s rsassa \
@@ -681,17 +778,28 @@ tpm2 clockrateadjust s -p "session:session.ctx"
 tpm2 flushcontext session.ctx
 
 # Test nvwrite
- create_authorized_policy
- tpm2 nvdefine 1 -s 8 -a "ownerread|authwrite|policywrite" -L authorized.policy
- echo "foo" | tpm2 nvwrite 1 -i- --cphash cp.hash
- xxd -p cp.hash
- generate_policycphash
- sign_and_verify_policycphash
- setup_authorized_policycphash
- echo "foo" | tpm2 nvwrite 1 -i- -P "session:session.ctx"
- tpm2 flushcontext session.ctx
+create_authorized_policy
+tpm2 nvdefine 1 -s 8 -a "ownerread|authwrite|policywrite" -L authorized.policy
 
- # Test encryptdecrypt
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+#
+# Suppose TPM2_NVDefine was not run and the name was entirely created external
+# to the TPM. Note: This support still needs to be added.
+#
+# To emulate the absence of NV index while calculating the cpHash of NVRead,
+# let's Undefine the NV index and define it later.
+#
+tpm2 nvundefine 1 -C o
+echo "foo" | tpm2 nvwrite 1 -i- -n $NV_INDEX_NAME --cphash cp.hash --tcti=none
+
+generate_policycphash
+sign_and_verify_policycphash
+tpm2 nvdefine 1 -s 8 -a "ownerread|authwrite|policywrite" -L authorized.policy
+setup_authorized_policycphash
+echo "foo" | tpm2 nvwrite 1 -i- -P "session:session.ctx"
+tpm2 flushcontext session.ctx
+
+# Test encryptdecrypt
 create_authorized_policy
 tpm2 createprimary -Q -C e -g sha1 -G rsa -c primary.ctx
 tpm2 create -Q -g sha256 -G aes -u key.pub -r key.priv -C primary.ctx \
