@@ -16,6 +16,9 @@ file_loadexternal_key_ctx=ctx_loadexternal_out_"$alg_primary_obj"_\
 "$alg_primary_key"-"$alg_create_obj"_"$alg_create_key"
 file_loadexternal_output=loadexternal_"$file_loadexternal_key_ctx"
 
+tss_privkey=tss_pk
+tss_prim=prim
+
 Handle_parent=0x81010019
 
 cleanup() {
@@ -235,5 +238,35 @@ run_rsa_passin_test "private.pem" "fd:42"
 echo -n "mypassword" > "passfile"
 run_rsa_passin_test "private.pem" "stdin" "passfile"
 
+#
+# Test cpHash output
+#
+tpm2 createprimary -C o -c prim.ctx -Q
+tpm2 create -C prim.ctx -u key.pub -r key.priv -Q
+tpm2 loadexternal -C n -u key.pub -c key.ctx --cphash cp.hash
+TPM2_CC_Loadexternal="00000167"
+External_priv_dat="0000"
+External_pub_dat=$(xxd -p key.pub | tr -d '\n')
+Hierarchy="40000007"
+echo -ne $TPM2_CC_Loadexternal$External_priv_dat$External_pub_dat$Hierarchy | \
+    xxd -r -p | openssl dgst -sha256 -binary -out test.bin
+cmp cp.hash test.bin 2
+if [ $? != 0 ]; then
+	echo "cpHash doesn't match calculated value"
+    exit 1
+fi
+
+#
+# TSS Privkey
+#
+tpm2 createprimary -G rsa -C o \
+-a "fixedtpm|fixedparent|sensitivedataorigin|userwithauth|restricted|decrypt|noda" \
+-c $tss_prim.ctx
+
+tpm2 create -C $tss_prim.ctx -u $tss_privkey.pub -r $tss_privkey.priv
+
+tpm2 encodeobject -C $tss_prim.ctx -u $tss_privkey.pub -r $tss_privkey.priv -o $tss_privkey.pem
+
+tpm2 loadexternal -r $tss_privkey.pem -c $tss_privkey.ctx
 
 exit 0
