@@ -11,6 +11,10 @@ ak_pubkey_file=ak.pub
 quote_file=quote.bin
 print_file=quote.yaml
 
+pem_file=tsskeypriv
+pem_pub=public_key
+tss_prim=prim
+
 cleanup() {
     rm -f $ak_name_file $ak_pubkey_file \
           $quote_file $print_file $ak_ctx \
@@ -41,6 +45,9 @@ yaml_verify $print_file
 tpm2 print -t TPMT_PUBLIC tpmt_public.ak > $print_file
 yaml_verify $print_file
 
+tpm2 print -t TPMT_PUBLIC -f pem tpmt_public.ak > $print_file
+openssl rsa -noout -text -inform PEM -in $print_file -pubin
+
 # Take PCR quote
 tpm2 quote -Q -c $ak_ctx -l "sha256:0,2,4,9,10,11,12,17" -q "0f8beb45ac" \
 -m $quote_file
@@ -90,4 +97,20 @@ if [ $? -eq 0 ]; then
   exit 1
 fi
 
+# TSS Privkey
+tpm2 createprimary -G rsa -C o \
+-a "fixedtpm|fixedparent|sensitivedataorigin|userwithauth|restricted|decrypt|noda" \
+-c $tss_prim.ctx
+
+tpm2 create -C $tss_prim.ctx -u $pem_file.pub -r $pem_file.priv
+
+tpm2 encodeobject -C $tss_prim.ctx -u $pem_file.pub -r $pem_file.priv \
+-o $pem_file.pem
+
+tpm2 print -t TSSPRIVKEY_OBJ $pem_file.pem
+
+tpm2 print -t TSSPRIVKEY_OBJ -f pem $pem_file.pem > $pem_pub.pem
+
+openssl rsa -pubin -in $pem_pub.pem -text
+
 exit 0
